from typing import List, Dict, Tuple, Optional
import re
from objects import REG, DFA, Terminal


# TODO ask: commas at the ends of lines or not?
class Parser:
    def __init__(self):
        pass

    def reg_to_str(self, reg: REG, full: bool = False) -> str:
        rules = self.rules_to_str(reg.rules)
        if not full:
            return "P=" + rules

        out = "(("
        for nonterminal in reg.nonterminals:
            out += nonterminal.name
            out += (",")

        out = out[:-1] + ("),(")
        for terminal in reg.terminals:
            out += (terminal.name)
            out += (",")

        out = out[:-1] + ("),P," + reg.init.name + ")\nP=")
        out += (self.rules_to_str(reg.rules))
        return out

    # TODO ask: import rules
    def rules_to_str(self, rules: REG.Rules):
        out = ""
        for rule in rules:
            out = out[:-1] + (",\n")
            rewrite = rules[rule]
            if len(rewrite) == 0:
                continue

            out += (rule.name + "->")
            for variant in rewrite:
                if type(variant) == Terminal:
                    out += variant.name
                else:
                    for i in range(len(variant)):
                        out += variant[i].name
                if next:  # TODO
                    out += "|"

        return "{" + out[2:len(out) - 1] + "}"

    #    def str_to_reg(self, source: str) -> REG:
    #        re.match(...((nt),(t),P,S)\n rest

    def dfa_to_str(self, dfa: DFA, full : bool = False) -> str:

        # full - verbose description of DFA - only for development, dismiss later

        states = "{"
        for state in dfa.states:
            states += state.name + ","
        states = states[:-1] + "}"

        # Vlada: states = ",".join(dfa.states) but I use it wrong or what

        characters = "{"
        for character in dfa.characters:
            characters += character.name + ","
        characters = characters[:-1] + "}"

        transition = ""
        for key, state_2 in dfa.transition.items():
            state_1, character = key
            transition += "(" + state_1.name + "," + character.name + ")=" + state_2.name + "\n"
        transition = "{" + transition[:-1] + "}"

        init = dfa.init.name

        final = "F={"
        for state in dfa.final:
            final += state.name + ","
        final = final[:-1] + "}"

        if full:
            return "DFA = (" + states + "," + characters + ",d," + \
                   init + "," + final + ")\n" + "d = " + transition

        else:
            return transition + "\n" + final


    def nfa_to_str(self, dfa: DFA, full : bool = False) -> str:

        # full - verbose description of DFA - only for development, dismiss later

        states = "{"
        for state in dfa.states:
            states += state.name + ","
        states = states[:-1] + "}"

        # Vlada: states = ",".join(dfa.states) but I use it wrong or what

        characters = "{"
        for character in dfa.characters:
            characters += character.name + ","
        characters = characters[:-1] + "}"

        transition = ""
        for key, set_states in dfa.transition.items():
            state_1, terminal = key
            transition += "(" + state_1.name + "," + terminal.name + ")={"
            for state in set_states:
                transition += state.name + ","
            transition = transition[:-1] + "}\n"
        transition = "{" + transition[:-1] + "}"

        init = dfa.init.name

        final = "F={"
        for state in dfa.final:
            final += state.name + ","
        final = final[:-1] + "}"

        if full:
            return "NFA = (" + states + "," + characters + ",d," + \
                   init + "," + final + ")\n" + "d = " + transition

        else:
            return transition + "\n" + final

# TODO DFA/NFA to string are too similar

# all: string <-> formal object
# reg
# dfa
# nfa
# pda
# cfg
